@@ -125,6 +125,9 @@ GST_START_TEST (test_get_bus)
   ASSERT_OBJECT_REFCOUNT (pipeline, "pipeline after get_bus", 1);
   ASSERT_OBJECT_REFCOUNT (bus, "bus", 2);
 
+  /* bindings don't like the floating flag to be set here */
+  fail_if (g_object_is_floating (bus));
+
   gst_object_unref (pipeline);
 
   ASSERT_OBJECT_REFCOUNT (bus, "bus after unref pipeline", 1);
@@ -238,7 +241,7 @@ static GMutex *probe_lock;
 static GCond *probe_cond;
 
 static gboolean
-sink_pad_probe (GstPad * pad, GstBuffer * buffer,
+sink_pad_probe (GstPad * pad, GstPadProbeType type, GstBuffer * buffer,
     GstClockTime * first_timestamp)
 {
   fail_if (GST_BUFFER_TIMESTAMP (buffer) == GST_CLOCK_TIME_NONE,
@@ -274,7 +277,8 @@ GST_START_TEST (test_base_time)
   gst_element_link (fakesrc, fakesink);
 
   sink = gst_element_get_static_pad (fakesink, "sink");
-  gst_pad_add_buffer_probe (sink, G_CALLBACK (sink_pad_probe), &observed);
+  gst_pad_add_probe (sink, GST_PAD_PROBE_TYPE_BUFFER,
+      (GstPadProbeCallback) sink_pad_probe, &observed, NULL);
 
   fail_unless (gst_element_set_state (pipeline, GST_STATE_PAUSED)
       == GST_STATE_CHANGE_NO_PREROLL, "expected no-preroll from live pipeline");
@@ -528,6 +532,38 @@ GST_START_TEST (test_concurrent_create)
 
 GST_END_TEST;
 
+GST_START_TEST (test_pipeline_in_pipeline)
+{
+  GstElement *pipeline, *bin, *fakesrc, *fakesink;
+  GstMessage *msg;
+
+  pipeline = gst_element_factory_make ("pipeline", "pipeline");
+  bin = gst_element_factory_make ("pipeline", "pipeline-as-bin");
+  fakesrc = gst_element_factory_make ("fakesrc", "fakesrc");
+  fakesink = gst_element_factory_make ("fakesink", "fakesink");
+
+  fail_unless (pipeline && bin && fakesrc && fakesink);
+
+  g_object_set (fakesrc, "num-buffers", 100, NULL);
+
+  gst_bin_add (GST_BIN (pipeline), bin);
+  gst_bin_add_many (GST_BIN (bin), fakesrc, fakesink, NULL);
+  gst_element_link (fakesrc, fakesink);
+
+  fail_unless_equals_int (gst_element_set_state (pipeline, GST_STATE_PLAYING),
+      GST_STATE_CHANGE_ASYNC);
+
+  msg = gst_bus_timed_pop_filtered (GST_ELEMENT_BUS (pipeline), -1,
+      GST_MESSAGE_EOS);
+  gst_message_unref (msg);
+
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_pipeline_suite (void)
 {
@@ -544,6 +580,7 @@ gst_pipeline_suite (void)
   tcase_add_test (tc_chain, test_bus);
   tcase_add_test (tc_chain, test_base_time);
   tcase_add_test (tc_chain, test_concurrent_create);
+  tcase_add_test (tc_chain, test_pipeline_in_pipeline);
 
   return s;
 }
