@@ -27,24 +27,9 @@
 typedef GstElement GstDummyEnc;
 typedef GstElementClass GstDummyEncClass;
 
-static void gst_dummy_enc_add_interfaces (GType enc_type);
-
 GType gst_dummy_enc_get_type (void);
-GST_BOILERPLATE_FULL (GstDummyEnc, gst_dummy_enc, GstElement,
-    GST_TYPE_ELEMENT, gst_dummy_enc_add_interfaces);
-
-static void
-gst_dummy_enc_add_interfaces (GType enc_type)
-{
-  static const GInterfaceInfo tag_setter_info = { NULL, NULL, NULL };
-
-  g_type_add_interface_static (enc_type, GST_TYPE_TAG_SETTER, &tag_setter_info);
-}
-
-static void
-gst_dummy_enc_base_init (gpointer g_class)
-{
-}
+G_DEFINE_TYPE_WITH_CODE (GstDummyEnc, gst_dummy_enc,
+    GST_TYPE_ELEMENT, G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL));
 
 static void
 gst_dummy_enc_class_init (GstDummyEncClass * klass)
@@ -52,7 +37,7 @@ gst_dummy_enc_class_init (GstDummyEncClass * klass)
 }
 
 static void
-gst_dummy_enc_init (GstDummyEnc * enc, GstDummyEncClass * klass)
+gst_dummy_enc_init (GstDummyEnc * enc)
 {
 }
 
@@ -105,13 +90,13 @@ GST_START_TEST (test_merge)
 
   setter = GST_TAG_SETTER (enc);
 
-  list1 = gst_tag_list_new ();
+  list1 = gst_tag_list_new_empty ();
   gst_tag_list_add (list1, GST_TAG_MERGE_APPEND, GST_TAG_ARTIST, "artist1",
       NULL);
   gst_tag_setter_merge_tags (setter, list1, GST_TAG_MERGE_APPEND);
   assert_tag_setter_list_length (setter, 1);
 
-  list2 = gst_tag_list_new ();
+  list2 = gst_tag_list_new_empty ();
   gst_tag_list_add (list2, GST_TAG_MERGE_APPEND, GST_TAG_ARTIST, "artist2",
       GST_TAG_TITLE, "title1", NULL);
   gst_tag_setter_merge_tags (setter, list2, GST_TAG_MERGE_APPEND);
@@ -127,8 +112,8 @@ GST_START_TEST (test_merge)
       NULL);
   assert_tag_setter_list_length (setter, 2);
 
-  gst_tag_list_free (list2);
-  gst_tag_list_free (list1);
+  gst_tag_list_unref (list2);
+  gst_tag_list_unref (list1);
 
   g_object_unref (enc);
 }
@@ -150,8 +135,8 @@ GST_START_TEST (test_merge_modes)
       fail_unless (enc != NULL);
 
       setter = GST_TAG_SETTER (enc);
-      list1 = gst_tag_list_new ();
-      list2 = gst_tag_list_new ();
+      list1 = gst_tag_list_new_empty ();
+      list2 = gst_tag_list_new_empty ();
 
       /* i = 0: -     -
        * i = 1: list1 -
@@ -175,9 +160,9 @@ GST_START_TEST (test_merge_modes)
       fail_unless_equals_int (tag_list_length (gst_tag_setter_get_tag_list
               (setter)), tag_list_length (merged));
 
-      gst_tag_list_free (list1);
-      gst_tag_list_free (list2);
-      gst_tag_list_free (merged);
+      gst_tag_list_unref (list1);
+      gst_tag_list_unref (list2);
+      gst_tag_list_unref (merged);
       gst_object_unref (enc);
     }
   }
@@ -200,8 +185,8 @@ GST_START_TEST (test_merge_modes_skip_empty)
       fail_unless (enc != NULL);
 
       setter = GST_TAG_SETTER (enc);
-      list1 = gst_tag_list_new ();
-      list2 = gst_tag_list_new ();
+      list1 = gst_tag_list_new_empty ();
+      list2 = gst_tag_list_new_empty ();
 
       if (i == 1) {
         gst_tag_list_add (list2, GST_TAG_MERGE_APPEND, GST_TAG_ARTIST,
@@ -215,9 +200,9 @@ GST_START_TEST (test_merge_modes_skip_empty)
       fail_unless_equals_int (tag_list_length (gst_tag_setter_get_tag_list
               (setter)), tag_list_length (merged));
 
-      gst_tag_list_free (list1);
-      gst_tag_list_free (list2);
-      gst_tag_list_free (merged);
+      gst_tag_list_unref (list1);
+      gst_tag_list_unref (list2);
+      gst_tag_list_unref (merged);
       gst_object_unref (enc);
     }
   }
